/*
 * vm-restart - simple daemon to detect VMware guest OS restart
 *
 * Copyright (c) 2009, Paul Chakravarti paul.chakravarti@gmail.com   
 *
 * Permission to use, copy, modify, and/or distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 *      Uses VMWare backdoor code from VM Back - VMware Command Line Tools
 *      http://chitchat.at.infoseek.co.jp/vmware/vmtools.html
 *
 *      Copyright (c) 2006 Ken Kato
 */

#include <stdint.h>

/*
	data struct for register values

	DO NOT change the order of members because it is hard-coded
	in the assembly portion of the code

	use uint8_t[4] for members instead of uint32_t in order to
	avoid dealing with compiler's struct alignment
*/

typedef struct _reg {
	uint8_t eax[4];
	uint8_t ebx[4];
	uint8_t ecx[4];
	uint8_t edx[4];
	uint8_t ebp[4];
	uint8_t edi[4];
	uint8_t esi[4];
} reg_t;

/* member accessing macros */

#define R_EAX(r)	*((uint32_t *)&((r).eax))
#define R_EBX(r)	*((uint32_t *)&((r).ebx))
#define R_ECX(r)	*((uint32_t *)&((r).ecx))
#define R_EDX(r)	*((uint32_t *)&((r).edx))
#define R_EBP(r)	*((uint32_t *)&((r).ebp))
#define R_EDI(r)	*((uint32_t *)&((r).edi))
#define R_ESI(r)	*((uint32_t *)&((r).esi))

/*	access command port with 'in' instruction */
void vmcall_cmd(reg_t *reg);

#define VMWARE_MAGIC		0x564D5868UL	/* Beckdoor magic number (VMXh)	*/
#define VMWARE_CMD_PORT		0x5658			/* Backdoor command port (VX)	*/
#define VMWARE_RPC_PORT		0x5659			/* RPC data transfer port (VY)	*/

/*
	backdoor command numbers
*/
#define VMCMD_GET_VERSION	0x0a			/* Get version number			*/
#define VMCMD_GET_GMTIME	0x17			/* Get host's time (GMT)		*/

#define VMTOOL_SUCCESS		0				/* success						*/
#define VMTOOL_CMD_ERROR	1				/* backdoor call error			*/

/*
	check virtual environment with command 0x0a
*/
int VMCheckVirtual(void);
/*
	get host system time with command 0x17
*/
int	VMGetGmTime(uint32_t *time, uint32_t *micro, uint32_t *whatsthis, int32_t *offset);

