/*
 * vm-restart - simple daemon to detect VMware guest OS restart
 *
 * Copyright (c) 2009, Paul Chakravarti paul.chakravarti@gmail.com   
 *
 * Permission to use, copy, modify, and/or distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 *      Uses VMWare backdoor code from VM Back - VMware Command Line Tools
 *      http://chitchat.at.infoseek.co.jp/vmware/vmtools.html
 *
 *      Copyright (c) 2006 Ken Kato
 */

#include "vm.h"

#include <setjmp.h>
#include <signal.h>
#include <time.h>

static sigjmp_buf	sigsegvbuffer;

static void sigsegvhandler(int sig)
{
	siglongjmp(sigsegvbuffer, sig);
}

int VMCheckVirtual()
{
	reg_t reg = {0};
	struct sigaction newaction, oldaction;

	R_EAX(reg) = VMWARE_MAGIC;
	R_EBX(reg) = ~VMWARE_MAGIC;
	R_ECX(reg) = VMCMD_GET_VERSION;
	R_EDX(reg) = VMWARE_CMD_PORT;

	sigemptyset(&newaction.sa_mask);
	newaction.sa_flags = SA_RESTART;
	newaction.sa_handler = sigsegvhandler;
	sigaction(SIGSEGV, &newaction, &oldaction);

	if (sigsetjmp(sigsegvbuffer, 1) != 0) {
		/* SIGSEGV is caught during the backdoor access */
		sigaction(SIGSEGV, &oldaction, 0);
		return VMTOOL_CMD_ERROR;
	}

	vmcall_cmd(&reg);

	sigaction(SIGSEGV, &oldaction, 0);

	if (R_EBX(reg) != VMWARE_MAGIC) {
		return VMTOOL_CMD_ERROR;
	}

	return VMTOOL_SUCCESS;
}

int VMGetGmTime(
	uint32_t *time, uint32_t *micro, uint32_t *whatsthis, int32_t *offset)
{
	reg_t reg = {0};

	R_EAX(reg) = VMWARE_MAGIC;
	R_ECX(reg) = VMCMD_GET_GMTIME;
	R_EDX(reg) = VMWARE_CMD_PORT;

	vmcall_cmd(&reg);

	if (!gmtime((time_t *)&R_EAX(reg))) {
		return VMTOOL_CMD_ERROR;
	}

	if (time) {
		*time = R_EAX(reg);
	}
	if (micro) {
		*micro = R_EBX(reg);
	}
	if (whatsthis) {
		*whatsthis = R_ECX(reg);
	}
	if (offset) {
		*offset = R_EDX(reg);
	}

	return VMTOOL_SUCCESS;
}

